<?php

namespace App\Controllers;

use App\Core\Database;
use App\Services\AuthService;
use App\Services\LogService;
use App\Services\LanguageService;
use App\Utils\DarkMode;

/**
 * Contrôleur pour la gestion des utilisateurs.
 * (Création, suppression, affichage).
 * Nécessite un rôle 'admin'.
 */
class UserController
{
    private $db;
    private $authService;
    private $logService;
    private $langService;

    /**
     * Constructeur du UserController.
     */
    public function __construct()
    {
        $this->db = Database::getInstance();
        $this->authService = new AuthService($this->db);
        $this->logService = new LogService(APP_LOG_PATH);
        $this->langService = new LanguageService(APP_ROOT_DIR . '/src/Lang/');
    }

    /**
     * Vérifie si l'utilisateur est un administrateur.
     * Si non, redirige et affiche un message d'erreur.
     */
    private function requireAdmin()
    {
        if (!$this->authService->isLoggedIn() || $this->authService->getUserRole() !== 'admin') {
            $_SESSION['error'] = $this->langService->__('permission_denied');
            header('Location: /dashboard');
            exit();
        }
    }

    /**
     * Affiche la liste des utilisateurs.
     */
    public function index()
    {
        $this->requireAdmin();

        global $translations;
        $currentLang = $this->langService->getLanguage();
        $darkModeClass = DarkMode::getBodyClass();

        $users = $this->db->query("SELECT id, username, role, created_at FROM users ORDER BY username ASC")->fetchAll();

        $successMessage = $_SESSION['success'] ?? null;
        unset($_SESSION['success']);
        $errorMessage = $_SESSION['error'] ?? null;
        unset($_SESSION['error']);

        $authService = $this->authService; // Add this line
        require_once APP_ROOT_DIR . '/src/Views/users/index.php';
    }

    /**
     * Affiche le formulaire de création d'un nouvel utilisateur.
     */
    public function showCreateForm()
    {
        $this->requireAdmin();

        global $translations;
        $currentLang = $this->langService->getLanguage();
        $darkModeClass = DarkMode::getBodyClass();

        $errorMessage = $_SESSION['error'] ?? null;
        unset($_SESSION['error']);

        require_once APP_ROOT_DIR . '/src/Views/users/create.php';
    }

    /**
     * Traite la soumission du formulaire de création d'utilisateur.
     */
    public function create()
    {
        $this->requireAdmin();

        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            header('Location: /users/create');
            exit();
        }

        $username = trim($_POST['username'] ?? '');
        $password = $_POST['password'] ?? '';
        $role = $_POST['role'] ?? 'user';
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $adminUserId = $this->authService->getUserId();

        if (empty($username) || empty($password)) {
            $_SESSION['error'] = $this->langService->__('user_create_error_empty_fields');
            header('Location: /users/create');
            exit();
        }
        if (!in_array($role, ['admin', 'user'])) {
            $_SESSION['error'] = $this->langService->__('user_create_error_invalid_role');
            header('Location: /users/create');
            exit();
        }

        // Vérifier si l'utilisateur existe déjà
        $stmt = $this->db->prepare("SELECT COUNT(*) FROM users WHERE username = ?");
        $stmt->execute([$username]);
        if ($stmt->fetchColumn() > 0) {
            $_SESSION['error'] = $this->langService->__('user_create_error_exists', ['username' => htmlspecialchars($username)]);
            header('Location: /users/create');
            exit();
        }

        // Hacher le mot de passe
        $hashedPassword = password_hash($password, PASSWORD_DEFAULT);

        try {
            $stmt = $this->db->prepare("INSERT INTO users (username, password, role) VALUES (?, ?, ?)");
            $stmt->execute([$username, $hashedPassword, $role]);
            $newUserId = $this->db->lastInsertId();

            $this->logService->log('info', "Nouvel utilisateur '{$username}' ({$role}) créé par l'administrateur.", $adminUserId, $ipAddress);
            $_SESSION['success'] = $this->langService->__('user_create_success', ['username' => htmlspecialchars($username)]);
        } catch (\PDOException $e) {
            error_log("Erreur lors de la création de l'utilisateur: " . $e->getMessage());
            $_SESSION['error'] = $this->langService->__('user_create_error_db');
            $this->logService->log('error', "Échec création utilisateur '{$username}': " . $e->getMessage(), $adminUserId, $ipAddress);
        }

        header('Location: /users');
        exit();
    }

    /**
     * Supprime un utilisateur.
     */
    public function delete()
    {
        $this->requireAdmin();

        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            header('Location: /users');
            exit();
        }

        $userIdToDelete = $_POST['user_id'] ?? null;
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $adminUserId = $this->authService->getUserId();

        if (empty($userIdToDelete)) {
            $_SESSION['error'] = $this->langService->__('user_delete_error_id_missing');
            header('Location: /users');
            exit();
        }

        // Empêcher un admin de se supprimer lui-même (ou le dernier admin)
        if ($userIdToDelete == $adminUserId) {
            $_SESSION['error'] = $this->langService->__('user_delete_error_self_delete');
            header('Location: /users');
            exit();
        }

        // Vérifier s'il reste au moins un administrateur après la suppression
        $stmt = $this->db->prepare("SELECT role FROM users WHERE id = ?");
        $stmt->execute([$userIdToDelete]);
        $userRoleToDelete = $stmt->fetchColumn();

        if ($userRoleToDelete === 'admin') {
            $stmt = $this->db->query("SELECT COUNT(*) FROM users WHERE role = 'admin'");
            $adminCount = $stmt->fetchColumn();
            if ($adminCount <= 1) {
                $_SESSION['error'] = $this->langService->__('user_delete_error_last_admin');
                header('Location: /users');
                exit();
            }
        }


        try {
            // Récupérer le nom d'utilisateur avant suppression pour le log
            $stmt = $this->db->prepare("SELECT username FROM users WHERE id = ?");
            $stmt->execute([$userIdToDelete]);
            $usernameToDelete = $stmt->fetchColumn();

            $stmt = $this->db->prepare("DELETE FROM users WHERE id = ?");
            $stmt->execute([$userIdToDelete]);

            if ($stmt->rowCount() > 0) {
                $this->logService->log('info', "Utilisateur '{$usernameToDelete}' (ID: {$userIdToDelete}) supprimé par l'administrateur.", $adminUserId, $ipAddress);
                $_SESSION['success'] = $this->langService->__('user_delete_success', ['username' => htmlspecialchars($usernameToDelete)]);
            } else {
                $_SESSION['error'] = $this->langService->__('user_delete_error_not_found');
            }
        } catch (\PDOException $e) {
            error_log("Erreur lors de la suppression de l'utilisateur: " . $e->getMessage());
            $_SESSION['error'] = $this->langService->__('user_delete_error_db');
            $this->logService->log('error', "Échec suppression utilisateur ID: {$userIdToDelete}: " . $e->getMessage(), $adminUserId, $ipAddress);
        }

        header('Location: /users');
        exit();
    }
}
